import { Calendar, OccurrenceGenerator } from '@rschedule/core/generators';
import {
  ParseJSONError,
  serializeDataToJSON,
  registerJSONSerializerFn,
} from '@rschedule/json-tools';

Calendar.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'Calendar') {
    throw new ParseJSONError('Invalid Calendar JSON');
  }
  return new Calendar({
    schedules: json.schedules.map(sched =>
      OccurrenceGenerator.fromJSON(sched, Object.assign({ timezone: json.timezone }, options)),
    ),
    data: typeof options.data === 'function' ? options.data(json) : json.data,
    timezone: options.timezone || json.timezone,
  });
};
Calendar.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'Calendar',
    schedules: this.schedules.map(schedule =>
      schedule.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
    ),
  };
  if (!opts.nested) json.timezone = this.timezone;
  serializeDataToJSON(this, json, opts);
  return json;
};
registerJSONSerializerFn('Calendar', Calendar.fromJSON);
