import { IntersectionOperator, OccurrenceGenerator } from '@rschedule/core/generators';
import { ParseJSONError, registerJSONSerializerFn } from '@rschedule/json-tools';

IntersectionOperator.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'IntersectionOperator',
    streams: this.streams.map(stream =>
      stream.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
    ),
  };
  if (this.config.base) {
    json.base = this.config.base.toJSON(opts);
  }
  if (!opts.nested && !this.config.base) json.timezone = this.timezone;
  if (this.maxFailedIterations) json.maxFailedIterations = this.maxFailedIterations;
  return json;
};
IntersectionOperator.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'IntersectionOperator') {
    throw new ParseJSONError('Invalid IntersectionOperator JSON.');
  }
  const base = json.base && OccurrenceGenerator.fromJSON(json.base, options);
  const timezone =
    // prettier-ignore
    options.timezone !== undefined ? options.timezone :
        (base && base.timezone) !== undefined ? base.timezone :
            json.timezone;
  if (timezone === undefined) {
    throw new ParseJSONError('Invalid IntersectionOperator JSON. Timezone is missing.');
  }
  return new IntersectionOperator(
    {
      maxFailedIterations: json.maxFailedIterations,
      streams: json.streams.map(stream =>
        OccurrenceGenerator.fromJSON(stream, Object.assign({ timezone }, options)),
      ),
    },
    {
      base,
      timezone,
    },
  );
};
registerJSONSerializerFn('IntersectionOperator', IntersectionOperator.fromJSON);
