import { MergeDurationOperator, OccurrenceGenerator } from '@rschedule/core/generators';
import { ParseJSONError, registerJSONSerializerFn } from '@rschedule/json-tools';

MergeDurationOperator.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'MergeDurationOperator',
    maxDuration: this.maxDuration,
  };
  if (this.config.base) {
    json.base = this.config.base.toJSON(opts);
  }
  if (!opts.nested && !this.config.base) json.timezone = this.timezone;
  return json;
};
MergeDurationOperator.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'MergeDurationOperator') {
    throw new ParseJSONError('Invalid MergeDurationOperator JSON.');
  }
  const base = json.base && OccurrenceGenerator.fromJSON(json.base, options);
  const timezone =
    // prettier-ignore
    options.timezone !== undefined ? options.timezone :
        (base && base.timezone) !== undefined ? base.timezone :
            json.timezone;
  if (timezone === undefined) {
    throw new ParseJSONError('Invalid MergeDurationOperator JSON. Timezone is missing.');
  }
  return new MergeDurationOperator(
    {
      maxDuration: json.maxDuration,
    },
    {
      base,
      timezone,
    },
  );
};
registerJSONSerializerFn('MergeDurationOperator', MergeDurationOperator.fromJSON);
