import { cloneJSON, DateAdapterBase } from '@rschedule/core';
import { Rule } from '@rschedule/core/generators';
import { serializeDataToJSON, registerJSONSerializerFn } from '@rschedule/json-tools';

Rule.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'Rule',
    config: Object.assign(Object.assign({}, cloneJSON(this.options)), {
      start: this.normalizeDateInput(this.options.start).toJSON(),
    }),
  };
  if (this.options.end) {
    json.config.end = this.normalizeDateInput(this.options.end).toJSON();
  }
  if (!opts.nested) json.timezone = this.timezone;
  serializeDataToJSON(this, json, opts);
  return json;
};
Rule.fromJSON = function fromJSON(json, options = {}) {
  const config = Object.assign(Object.assign({}, json.config), {
    start: DateAdapterBase.adapter.fromJSON(json.config.start),
    end: json.config.end && DateAdapterBase.adapter.fromJSON(json.config.end),
  });
  return new Rule(config, {
    data: typeof options.data === 'function' ? options.data(json) : json.data,
    timezone: options.timezone || json.timezone,
  });
};
registerJSONSerializerFn('Rule', Rule.fromJSON);
