import { Dates, Rule, Schedule } from '@rschedule/core/generators';
import {
  serializeDataToJSON,
  ParseJSONError,
  registerJSONSerializerFn,
} from '@rschedule/json-tools';
import { DateAdapterBase, cloneJSON } from '@rschedule/core';

Dates.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'Dates',
    dates: this.adapters.map(adapter => adapter.toJSON()),
  };
  if (!opts.nested) json.timezone = this.timezone;
  serializeDataToJSON(this, json, opts);
  return json;
};
Dates.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'Dates') {
    throw new ParseJSONError('Invalid Dates JSON');
  }
  return new Dates({
    dates: json.dates.map(date => DateAdapterBase.adapter.fromJSON(date)),
    data: typeof options.data === 'function' ? options.data(json) : json.data,
    timezone: options.timezone || json.timezone,
  });
};
registerJSONSerializerFn('Dates', Dates.fromJSON);

Rule.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'Rule',
    config: Object.assign(Object.assign({}, cloneJSON(this.options)), {
      start: this.normalizeDateInput(this.options.start).toJSON(),
    }),
  };
  if (this.options.end) {
    json.config.end = this.normalizeDateInput(this.options.end).toJSON();
  }
  if (!opts.nested) json.timezone = this.timezone;
  serializeDataToJSON(this, json, opts);
  return json;
};
Rule.fromJSON = function fromJSON(json, options = {}) {
  const config = Object.assign(Object.assign({}, json.config), {
    start: DateAdapterBase.adapter.fromJSON(json.config.start),
    end: json.config.end && DateAdapterBase.adapter.fromJSON(json.config.end),
  });
  return new Rule(config, {
    data: typeof options.data === 'function' ? options.data(json) : json.data,
    timezone: options.timezone || json.timezone,
  });
};
registerJSONSerializerFn('Rule', Rule.fromJSON);

Schedule.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'Schedule',
    rrules: this.rrules.map(rule =>
      rule.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
    ),
    exrules: this.exrules.map(rule =>
      rule.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
    ),
    rdates: this.rdates.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
    exdates: this.exdates.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
  };
  if (!opts.nested) json.timezone = this.timezone;
  serializeDataToJSON(this, json, opts);
  return json;
};
Schedule.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'Schedule') {
    throw new ParseJSONError('Invalid Schedule JSON');
  }
  return new Schedule({
    rrules: json.rrules.map(rule =>
      Rule.fromJSON(rule, Object.assign({ timezone: json.timezone }, options)),
    ),
    exrules: json.exrules.map(rule =>
      Rule.fromJSON(rule, Object.assign({ timezone: json.timezone }, options)),
    ),
    rdates: Dates.fromJSON(json.rdates, Object.assign({ timezone: json.timezone }, options)),
    exdates: Dates.fromJSON(json.exdates, Object.assign({ timezone: json.timezone }, options)),
    data: typeof options.data === 'function' ? options.data(json) : json.data,
    timezone: options.timezone || json.timezone,
  });
};
registerJSONSerializerFn('Schedule', Schedule.fromJSON);
