import { SubtractOperator, OccurrenceGenerator } from '@rschedule/core/generators';
import { ParseJSONError, registerJSONSerializerFn } from '@rschedule/json-tools';

SubtractOperator.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'SubtractOperator',
    streams: this.streams.map(stream =>
      stream.toJSON(Object.assign(Object.assign({}, opts), { nested: true })),
    ),
  };
  if (this.config.base) {
    json.base = this.config.base.toJSON(opts);
  }
  if (!opts.nested && !this.config.base) json.timezone = this.timezone;
  return json;
};
SubtractOperator.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'SubtractOperator') {
    throw new ParseJSONError('Invalid SubtractOperator JSON.');
  }
  const base = json.base && OccurrenceGenerator.fromJSON(json.base, options);
  const timezone =
    // prettier-ignore
    options.timezone !== undefined ? options.timezone :
        (base && base.timezone) !== undefined ? base.timezone :
            json.timezone;
  if (timezone === undefined) {
    throw new ParseJSONError('Invalid SubtractOperator JSON. Timezone is missing.');
  }
  return new SubtractOperator(
    json.streams.map(stream =>
      OccurrenceGenerator.fromJSON(stream, Object.assign({ timezone }, options)),
    ),
    {
      base,
      timezone,
    },
  );
};
registerJSONSerializerFn('SubtractOperator', SubtractOperator.fromJSON);
