import { UniqueOperator, OccurrenceGenerator } from '@rschedule/core/generators';
import { ParseJSONError, registerJSONSerializerFn } from '@rschedule/json-tools';

UniqueOperator.prototype.toJSON = function serialize(opts = {}) {
  const json = {
    type: 'UniqueOperator',
    base: this.config.base.toJSON(opts),
  };
  if (!opts.nested && !this.config.base) json.timezone = this.timezone;
  return json;
};
UniqueOperator.fromJSON = function parse(json, options = {}) {
  if (json.type !== 'UniqueOperator') {
    throw new ParseJSONError('Invalid UniqueOperator JSON.');
  }
  const base = json.base && OccurrenceGenerator.fromJSON(json.base, options);
  const timezone =
    // prettier-ignore
    options.timezone !== undefined ? options.timezone :
        (base && base.timezone) !== undefined ? base.timezone :
            json.timezone;
  if (timezone === undefined) {
    throw new ParseJSONError('Invalid UniqueOperator JSON. Timezone is missing.');
  }
  if (!base) {
    throw new ParseJSONError('Invalid UniqueOperator JSON. "base" is missing.');
  }
  return new UniqueOperator([], {
    base,
    timezone,
  });
};
registerJSONSerializerFn('UniqueOperator', UniqueOperator.fromJSON);
