import { OccurrenceGenerator } from '@rschedule/core/generators';

class ParseJSONError extends Error {}
class SerializeJSONError extends Error {}
OccurrenceGenerator.JSON_FN_MAP = new Map();
OccurrenceGenerator.prototype.toJSON = function serialize(opts) {
  throw new SerializeJSONError(
    `To support smaller bundles, ${this.constructor.name}#toJSON() ` +
      `must be manually added. See "@rschedule/json-tools" in the rSchedule docs.`,
  );
};
OccurrenceGenerator.fromJSON = function parse(json, options) {
  if (typeof json !== 'object' || json === null) {
    throw new ParseJSONError(`Invalid json "${json}"`);
  }
  const fn = OccurrenceGenerator.JSON_FN_MAP.get(json.type);
  if (!fn) {
    throw new ParseJSONError(
      `Unknown rSchedule object type "${json.type}". Have you added a json serializer ` +
        `for "${json.type}"? ` +
        `See "@rschedule/json-tools" in the rSchedule docs.`,
    );
  }
  return fn(json, options);
};
function serializeDataToJSON(gen, json, opt) {
  if (!opt.data) return;
  if (opt.data === true) {
    json.data = gen.data;
  } else {
    json.data = opt.data(gen);
  }
  if (json.data === undefined) delete json.data;
}
function registerJSONSerializerFn(name, fn) {
  OccurrenceGenerator.JSON_FN_MAP.set(name, fn);
}

export { ParseJSONError, SerializeJSONError, registerJSONSerializerFn, serializeDataToJSON };
