(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(require('@rschedule/core/generators'), require('@rschedule/json-tools'))
    : typeof define === 'function' && define.amd
    ? define(['@rschedule/core/generators', '@rschedule/json-tools'], factory)
    : ((global = global || self), factory(global.rScheduleGenerators, global.rScheduleJSONTools));
})(this, function(generators, jsonTools) {
  'use strict';

  /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

  var __assign = function() {
    __assign =
      Object.assign ||
      function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
          s = arguments[i];
          for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
      };
    return __assign.apply(this, arguments);
  };

  generators.Calendar.fromJSON = function parse(json, options) {
    if (options === void 0) {
      options = {};
    }
    if (json.type !== 'Calendar') {
      throw new jsonTools.ParseJSONError('Invalid Calendar JSON');
    }
    return new generators.Calendar({
      schedules: json.schedules.map(function(sched) {
        return generators.OccurrenceGenerator.fromJSON(
          sched,
          __assign({ timezone: json.timezone }, options),
        );
      }),
      data: typeof options.data === 'function' ? options.data(json) : json.data,
      timezone: options.timezone || json.timezone,
    });
  };
  generators.Calendar.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'Calendar',
      schedules: this.schedules.map(function(schedule) {
        return schedule.toJSON(__assign(__assign({}, opts), { nested: true }));
      }),
    };
    if (!opts.nested) json.timezone = this.timezone;
    jsonTools.serializeDataToJSON(this, json, opts);
    return json;
  };
  jsonTools.registerJSONSerializerFn('Calendar', generators.Calendar.fromJSON);
});
