(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(
        require('@rschedule/core'),
        require('@rschedule/core/generators'),
        require('@rschedule/json-tools'),
      )
    : typeof define === 'function' && define.amd
    ? define(['@rschedule/core', '@rschedule/core/generators', '@rschedule/json-tools'], factory)
    : ((global = global || self),
      factory(global.rSchedule, global.rScheduleGenerators, global.rScheduleJSONTools));
})(this, function(core, generators, jsonTools) {
  'use strict';

  generators.Dates.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'Dates',
      dates: this.adapters.map(function(adapter) {
        return adapter.toJSON();
      }),
    };
    if (!opts.nested) json.timezone = this.timezone;
    jsonTools.serializeDataToJSON(this, json, opts);
    return json;
  };
  generators.Dates.fromJSON = function parse(json, options) {
    if (options === void 0) {
      options = {};
    }
    if (json.type !== 'Dates') {
      throw new jsonTools.ParseJSONError('Invalid Dates JSON');
    }
    return new generators.Dates({
      dates: json.dates.map(function(date) {
        return core.DateAdapterBase.adapter.fromJSON(date);
      }),
      data: typeof options.data === 'function' ? options.data(json) : json.data,
      timezone: options.timezone || json.timezone,
    });
  };
  jsonTools.registerJSONSerializerFn('Dates', generators.Dates.fromJSON);
});
