(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(require('@rschedule/core/generators'), require('@rschedule/json-tools'))
    : typeof define === 'function' && define.amd
    ? define(['@rschedule/core/generators', '@rschedule/json-tools'], factory)
    : ((global = global || self), factory(global.rScheduleGenerators, global.rScheduleJSONTools));
})(this, function(generators, jsonTools) {
  'use strict';

  generators.MergeDurationOperator.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'MergeDurationOperator',
      maxDuration: this.maxDuration,
    };
    if (this.config.base) {
      json.base = this.config.base.toJSON(opts);
    }
    if (!opts.nested && !this.config.base) json.timezone = this.timezone;
    return json;
  };
  generators.MergeDurationOperator.fromJSON = function parse(json, options) {
    if (options === void 0) {
      options = {};
    }
    if (json.type !== 'MergeDurationOperator') {
      throw new jsonTools.ParseJSONError('Invalid MergeDurationOperator JSON.');
    }
    var base = json.base && generators.OccurrenceGenerator.fromJSON(json.base, options);
    var timezone =
      // prettier-ignore
      options.timezone !== undefined ? options.timezone :
          (base && base.timezone) !== undefined ? base.timezone :
              json.timezone;
    if (timezone === undefined) {
      throw new jsonTools.ParseJSONError(
        'Invalid MergeDurationOperator JSON. Timezone is missing.',
      );
    }
    return new generators.MergeDurationOperator(
      {
        maxDuration: json.maxDuration,
      },
      {
        base: base,
        timezone: timezone,
      },
    );
  };
  jsonTools.registerJSONSerializerFn(
    'MergeDurationOperator',
    generators.MergeDurationOperator.fromJSON,
  );
});
