(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(
        require('@rschedule/core/generators'),
        require('@rschedule/json-tools'),
        require('@rschedule/core'),
      )
    : typeof define === 'function' && define.amd
    ? define(['@rschedule/core/generators', '@rschedule/json-tools', '@rschedule/core'], factory)
    : ((global = global || self),
      factory(global.rScheduleGenerators, global.rScheduleJSONTools, global.rSchedule));
})(this, function(generators, jsonTools, core) {
  'use strict';

  /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

  var __assign = function() {
    __assign =
      Object.assign ||
      function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
          s = arguments[i];
          for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
      };
    return __assign.apply(this, arguments);
  };

  generators.Dates.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'Dates',
      dates: this.adapters.map(function(adapter) {
        return adapter.toJSON();
      }),
    };
    if (!opts.nested) json.timezone = this.timezone;
    jsonTools.serializeDataToJSON(this, json, opts);
    return json;
  };
  generators.Dates.fromJSON = function parse(json, options) {
    if (options === void 0) {
      options = {};
    }
    if (json.type !== 'Dates') {
      throw new jsonTools.ParseJSONError('Invalid Dates JSON');
    }
    return new generators.Dates({
      dates: json.dates.map(function(date) {
        return core.DateAdapterBase.adapter.fromJSON(date);
      }),
      data: typeof options.data === 'function' ? options.data(json) : json.data,
      timezone: options.timezone || json.timezone,
    });
  };
  jsonTools.registerJSONSerializerFn('Dates', generators.Dates.fromJSON);

  generators.Rule.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'Rule',
      config: __assign(__assign({}, core.cloneJSON(this.options)), {
        start: this.normalizeDateInput(this.options.start).toJSON(),
      }),
    };
    if (this.options.end) {
      json.config.end = this.normalizeDateInput(this.options.end).toJSON();
    }
    if (!opts.nested) json.timezone = this.timezone;
    jsonTools.serializeDataToJSON(this, json, opts);
    return json;
  };
  generators.Rule.fromJSON = function fromJSON(json, options) {
    if (options === void 0) {
      options = {};
    }
    var config = __assign(__assign({}, json.config), {
      start: core.DateAdapterBase.adapter.fromJSON(json.config.start),
      end: json.config.end && core.DateAdapterBase.adapter.fromJSON(json.config.end),
    });
    return new generators.Rule(config, {
      data: typeof options.data === 'function' ? options.data(json) : json.data,
      timezone: options.timezone || json.timezone,
    });
  };
  jsonTools.registerJSONSerializerFn('Rule', generators.Rule.fromJSON);

  generators.Schedule.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'Schedule',
      rrules: this.rrules.map(function(rule) {
        return rule.toJSON(__assign(__assign({}, opts), { nested: true }));
      }),
      exrules: this.exrules.map(function(rule) {
        return rule.toJSON(__assign(__assign({}, opts), { nested: true }));
      }),
      rdates: this.rdates.toJSON(__assign(__assign({}, opts), { nested: true })),
      exdates: this.exdates.toJSON(__assign(__assign({}, opts), { nested: true })),
    };
    if (!opts.nested) json.timezone = this.timezone;
    jsonTools.serializeDataToJSON(this, json, opts);
    return json;
  };
  generators.Schedule.fromJSON = function parse(json, options) {
    if (options === void 0) {
      options = {};
    }
    if (json.type !== 'Schedule') {
      throw new jsonTools.ParseJSONError('Invalid Schedule JSON');
    }
    return new generators.Schedule({
      rrules: json.rrules.map(function(rule) {
        return generators.Rule.fromJSON(rule, __assign({ timezone: json.timezone }, options));
      }),
      exrules: json.exrules.map(function(rule) {
        return generators.Rule.fromJSON(rule, __assign({ timezone: json.timezone }, options));
      }),
      rdates: generators.Dates.fromJSON(
        json.rdates,
        __assign({ timezone: json.timezone }, options),
      ),
      exdates: generators.Dates.fromJSON(
        json.exdates,
        __assign({ timezone: json.timezone }, options),
      ),
      data: typeof options.data === 'function' ? options.data(json) : json.data,
      timezone: options.timezone || json.timezone,
    });
  };
  jsonTools.registerJSONSerializerFn('Schedule', generators.Schedule.fromJSON);
});
