(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(require('@rschedule/core/generators'), require('@rschedule/json-tools'))
    : typeof define === 'function' && define.amd
    ? define(['@rschedule/core/generators', '@rschedule/json-tools'], factory)
    : ((global = global || self), factory(global.rScheduleGenerators, global.rScheduleJSONTools));
})(this, function(generators, jsonTools) {
  'use strict';

  generators.UniqueOperator.prototype.toJSON = function serialize(opts) {
    if (opts === void 0) {
      opts = {};
    }
    var json = {
      type: 'UniqueOperator',
      base: this.config.base.toJSON(opts),
    };
    if (!opts.nested && !this.config.base) json.timezone = this.timezone;
    return json;
  };
  generators.UniqueOperator.fromJSON = function parse(json, options) {
    if (options === void 0) {
      options = {};
    }
    if (json.type !== 'UniqueOperator') {
      throw new jsonTools.ParseJSONError('Invalid UniqueOperator JSON.');
    }
    var base = json.base && generators.OccurrenceGenerator.fromJSON(json.base, options);
    var timezone =
      // prettier-ignore
      options.timezone !== undefined ? options.timezone :
          (base && base.timezone) !== undefined ? base.timezone :
              json.timezone;
    if (timezone === undefined) {
      throw new jsonTools.ParseJSONError('Invalid UniqueOperator JSON. Timezone is missing.');
    }
    if (!base) {
      throw new jsonTools.ParseJSONError('Invalid UniqueOperator JSON. "base" is missing.');
    }
    return new generators.UniqueOperator([], {
      base: base,
      timezone: timezone,
    });
  };
  jsonTools.registerJSONSerializerFn('UniqueOperator', generators.UniqueOperator.fromJSON);
});
