(function(global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined'
    ? factory(exports, require('@rschedule/core/generators'))
    : typeof define === 'function' && define.amd
    ? define(['exports', '@rschedule/core/generators'], factory)
    : ((global = global || self),
      factory((global.rScheduleJSONTools = {}), global.rScheduleGenerators));
})(this, function(exports, generators) {
  'use strict';

  /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
  /* global Reflect, Promise */

  var extendStatics = function(d, b) {
    extendStatics =
      Object.setPrototypeOf ||
      ({ __proto__: [] } instanceof Array &&
        function(d, b) {
          d.__proto__ = b;
        }) ||
      function(d, b) {
        for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
      };
    return extendStatics(d, b);
  };

  function __extends(d, b) {
    extendStatics(d, b);
    function __() {
      this.constructor = d;
    }
    d.prototype = b === null ? Object.create(b) : ((__.prototype = b.prototype), new __());
  }

  var ParseJSONError = /** @class */ (function(_super) {
    __extends(ParseJSONError, _super);
    function ParseJSONError() {
      return (_super !== null && _super.apply(this, arguments)) || this;
    }
    return ParseJSONError;
  })(Error);
  var SerializeJSONError = /** @class */ (function(_super) {
    __extends(SerializeJSONError, _super);
    function SerializeJSONError() {
      return (_super !== null && _super.apply(this, arguments)) || this;
    }
    return SerializeJSONError;
  })(Error);
  generators.OccurrenceGenerator.JSON_FN_MAP = new Map();
  generators.OccurrenceGenerator.prototype.toJSON = function serialize(opts) {
    throw new SerializeJSONError(
      'To support smaller bundles, ' +
        this.constructor.name +
        '#toJSON() ' +
        'must be manually added. See "@rschedule/json-tools" in the rSchedule docs.',
    );
  };
  generators.OccurrenceGenerator.fromJSON = function parse(json, options) {
    if (typeof json !== 'object' || json === null) {
      throw new ParseJSONError('Invalid json "' + json + '"');
    }
    var fn = generators.OccurrenceGenerator.JSON_FN_MAP.get(json.type);
    if (!fn) {
      throw new ParseJSONError(
        'Unknown rSchedule object type "' +
          json.type +
          '". Have you added a json serializer ' +
          ('for "' + json.type + '"? ') +
          'See "@rschedule/json-tools" in the rSchedule docs.',
      );
    }
    return fn(json, options);
  };
  function serializeDataToJSON(gen, json, opt) {
    if (!opt.data) return;
    if (opt.data === true) {
      json.data = gen.data;
    } else {
      json.data = opt.data(gen);
    }
    if (json.data === undefined) delete json.data;
  }
  function registerJSONSerializerFn(name, fn) {
    generators.OccurrenceGenerator.JSON_FN_MAP.set(name, fn);
  }

  exports.ParseJSONError = ParseJSONError;
  exports.SerializeJSONError = SerializeJSONError;
  exports.registerJSONSerializerFn = registerJSONSerializerFn;
  exports.serializeDataToJSON = serializeDataToJSON;

  Object.defineProperty(exports, '__esModule', { value: true });
});
